import { ChangeDetectorRef, Component, ElementRef, EventEmitter, Input, Output, ViewChild } from '@angular/core';
import { ToastrService } from 'ngx-toastr';
import { StatusCodeModel } from 'src/app/shared/model/status-code.model';
import { EvaluationAssessmentService } from 'src/app/shared/services/evaluation-assessment.service';
import { EvaluationCycleService } from 'src/app/shared/services/evaluation-cycle.service';
export interface DataModel {
  evaluationRoundId: string;
  tdesc: string;
  edesc: string;
  apsyear: string;
  apsPeriodStart: string;
  apsPeriodEnd: string;
  statusCode: StatusCodeModel;
}
@Component({
  selector: 'app-edit-define-document-form',
  templateUrl: './edit-define-document-form.component.html',
  styleUrls: ['./edit-define-document-form.component.scss']
})

export class EditDefineDocumentFormComponent {
  @ViewChild('modalEdit') modalEdit!: ElementRef;
  @Input() evaluationRound?: DataModel;
  @Output() showFrom: EventEmitter<boolean> = new EventEmitter<boolean>();
  currentPage = 1;
  page = Array.from({ length: 1 }, (_, i) => i + 1);
  currentModal = "";
  search = '';

  assessmentList:any[]=[]
  loadingAssess= false

  evaluationRoundId = ""
  apsassessy = ""
  
  constructor(private toastr: ToastrService,
  private evaluationAssessmentService: EvaluationAssessmentService,
  private cdr: ChangeDetectorRef) { 
    
  }
ngOnInit(): void {
  if(this.evaluationRound){
    this.getEvaluationCycleList()
  }
}
saveAssessmentList(){
  let body = this.assessmentList.filter(x => x.active == true)
  if(body.length > 0){
    this.evaluationAssessmentService.postList(body).subscribe({
      next: response => {
        if (response.success) {
          this.showAlert(response.message, 'success')
          this.getEvaluationCycleList()
        } else {
  
          }
    }
    })
  }else{
    this.showAlert('ไม่พบรายการ', 'error')
  }
 
}
getApsassessy(){
  this.loadingAssess = true
  this.evaluationAssessmentService.getApsassessy(this.evaluationRoundId,this.apsassessy).subscribe({
    next: response => {
      
      this.loadingAssess = false
      this.cdr.detectChanges();
    },
    error: err => {
      this.loadingAssess = false
      console.error('Error fetching evaluation cycles:', err);
  }
})
}
searchChange() {
  this.currentPage = 1
  this.page = Array.from({ length: Math.ceil(this.assessmentListFilter().length / 10) }, (_, i) => i + 1);
}
getEvaluationCycleList() {
  this.evaluationAssessmentService.getById(this.evaluationRound!.evaluationRoundId).subscribe({
        next: response => {
          this.assessmentList = response
          this.cdr.detectChanges();
        },
        error: err => {
          console.error('Error fetching evaluation cycles:', err);
        }
      });
}
assessmentListFilter(){
  return this.assessmentList.filter(x =>
    x.apsassessy.fname.toLowerCase().includes(this.search.toLowerCase()) ||
    x.apsassessy.lname.toLowerCase().includes(this.search.toLowerCase()) ||
    x.apsassessy.efname.toLowerCase().includes(this.search.toLowerCase()) ||
    x.apsassessy.elname.toLowerCase().includes(this.search.toLowerCase()) ||
    x.apsassessy.employeeId.toLowerCase().includes(this.search.toLowerCase())
  )
}

  onBack() {
    this.showFrom.emit(false);
  }
  // การจัดการการเปิดปิด modal
  modalOptions: {
    [nameModal: string]: {
      isModalOpen: boolean;
      modalSize: string;
      backdropClose: boolean;
    };
  } = {
    "add": {
      isModalOpen: false,
      modalSize: 'm',
      backdropClose: true,
    },
    "edit": {
      isModalOpen: false,
      modalSize: 'm',
      backdropClose: true,
    }
  };
  openModalTest(id: string) {
    document.getElementById(id)?.classList.add('open');
    document.getElementById(id)?.classList.remove('hidden');
    setTimeout(() => {
      document.getElementById(id)?.setAttribute('aria-overlay', 'true')
      document.getElementById(id)?.setAttribute('tabindex', '0')
    }, 10);
    console.log("🚀 ~ EditDefineDocumentFormComponent ~ openModalTest ~ document.getElementById(id):", document.getElementById(id))

  }


  openEdit(evaluationRoundId:string,apsassessy:string) {
    if(evaluationRoundId&&apsassessy){
      const modalElement = this.modalEdit.nativeElement;
      setTimeout(() => {
        modalElement.setAttribute('aria-overlay', 'false');
        console.log("🚀 ~ EditDefineDocumentFormComponent ~ openEdit ~ modalElement:", modalElement)
      }, 10);
      this.evaluationRoundId = evaluationRoundId
      this.apsassessy = apsassessy
      this.getApsassessy()
    }
    
  }
  dataListCheck(){
   console.log( this.assessmentList);
   
  }
  openModal(name: string, size: string, closeOnBackdrop?: boolean) {
    this.modalOptions[name].modalSize = size;
    this.modalOptions[name].backdropClose = closeOnBackdrop || false;
    this.modalOptions[name].isModalOpen = true;
    this.currentModal = name;  // ตั้งค่าค่าของ currentModal เป็น 'add' หรือ 'edit'
    document.body.style.overflow = 'hidden';
  }

  closeModal(name: string) {
    this.modalOptions[name].isModalOpen = false;
    if (!this.isAnyModalOpen()) {
      document.body.style.overflow = ''; // คืนค่าการ Scroll เฉพาะเมื่อ Modal ทั้งหมดปิดแล้ว
    }
  }

  isAnyModalOpen(): boolean {
    return Object.values(this.modalOptions).some(modal => modal.isModalOpen); // ตรวจสอบว่า modal อื่นยังเปิดอยู่หรือไม่
  }

  // ฟังก์ชัน toggle checkbox
  isChecked: boolean = false; // ใช้สำหรับตรวจสอบสถานะของ checkbox
  
  toggleCheckbox() {
    this.isChecked = !this.isChecked; // สลับสถานะ
  }

  // ฟังก์ชันสำหรับการเพิ่ม ลบ หรือแก้ไข ข้อมูล
  addUser() {

  }
  showAlert(text: string, type: 'success' | 'error') {
    this.toastr[type](text, 'แจ้งเตือน', {
      timeOut: 3000,
      positionClass: 'toast-top-right',
    });
  }

  showSuccess() {
    this.toastr.success('บันทึกข้อมูลสำเร็จ', 'เเจ้งเตือน', {
      timeOut: 3000,
      positionClass: 'toast-top-right',
    });
  }

  showSuccessEdit() {
    this.toastr.success('เเก้ไขข้อมูลสำเร็จ', 'เเจ้งเตือน', {
      timeOut: 3000,
      positionClass: 'toast-top-right',
    });
  }

  showSuccessDelete() {
    this.toastr.success('ลบข้อมูลสำเร็จ', 'เเจ้งเตือน', {
      timeOut: 3000,
      positionClass: 'toast-top-right',
    });
  }
}
