import { ChangeDetectorRef, Component, EventEmitter, Output, Renderer2 } from '@angular/core';
import { ToastrService } from 'ngx-toastr';
import { EvaluationCycleModel, MyEvaluationCycleModel } from 'src/app/shared/model/evaluation-cycle.model';
import { EvaluationCycleService } from 'src/app/shared/services/evaluation-cycle.service';
import { MyStatusCodeModel, StatusCodeModel } from 'src/app/shared/model/status-code.model';
import { EvaluationAssessmentService } from 'src/app/shared/services/evaluation-assessment.service';

export interface DataModel {
  evaluationRoundId: string;
  tdesc: string;
  edesc: string;
  apsyear: string;
  apsPeriodStart: string;
  apsPeriodEnd: string;
  statusCode: StatusCodeModel;
}

@Component({
  selector: 'app-evaluation-cycle',
  templateUrl: './evaluation-cycle.component.html',
  styleUrls: ['./evaluation-cycle.component.scss']
})
export class EvaluationCycleComponent {
  @Output() evaluationRoundIdChange = new EventEmitter<string>();
  apsPeriodStart: string = '';
  apsPeriodEnd: string = '';

  currentPage = 1
  page = Array.from({ length: 1 }, (_, i) => i + 1);
  search = ""
  modalStatus = 'add'
  evaluation_cyclelist: { check: boolean, data: DataModel }[] = []
  evaluation_cycle: EvaluationCycleModel = new MyEvaluationCycleModel({})
  dataLoading = false
  dataSelect: DataModel = { evaluationRoundId: "", tdesc: "", edesc: "", apsyear: "", apsPeriodStart: "", apsPeriodEnd: "", statusCode: new MyStatusCodeModel({}) }
  itemToDelete: EvaluationCycleModel | null = null;
  isDataListChecked = false
  isDataListCheckedAll = false
  numDataListChecked = 0
  evaluationRoundId = ''
  private unlisten!: () => void;

  constructor(private evaluationCycleService: EvaluationCycleService,
    private toastr: ToastrService,
    private cdr: ChangeDetectorRef,
    private renderer: Renderer2
    
  ) { }

  ngOnInit(): void {
    this.getEvaluationCycleList()
    this.unlisten = this.renderer.listen('document', 'keydown', (event) => {
      if (event.key === 'Escape') {
          this.evaluationRoundId=''
      }
    });
  }
  ngOnDestroy() {
    if (this.unlisten) {
      this.unlisten(); // เรียกใช้งานจริง ๆ เพื่อลบ event listener
    }
  }
  getEvaluationCycleList() {
    this.dataLoading = true
    this.evaluationCycleService.getList().subscribe({
      next: response => {
        this.evaluation_cyclelist = response.map(x => ({
          check: false,
          data: {
            evaluationRoundId: x.evaluationRoundId,
            tdesc: x.tdesc,
            edesc: x.edesc,
            apsyear: x.apsyear,
            apsPeriodStart: x.apsPeriodStart,
            apsPeriodEnd: x.apsPeriodEnd,
            statusCode: new MyStatusCodeModel(x.statusCode || {})
          }
        }));

        this.dataLoading = false
        this.isDataListCheckedAll = false
        this.dataListCheckAll()
        this.searchChange();
        this.cdr.detectChanges();
      },
      error: err => {
        this.dataLoading = false
        this.cdr.detectChanges();
      }
    });
  }
  dataListCheckAll() {
    const selectAll = this.isDataListCheckedAll;
    this.evaluation_cycleListFilter().forEach(x => x.check = selectAll);
    this.dataListCheck();
  }

  dataListCheck() {
    const dataCheck = this.evaluation_cycleListFilter();
    this.isDataListCheckedAll = dataCheck.length ? dataCheck.every(x => x.check) : false;
    this.numDataListChecked = this.evaluation_cyclelist.filter(x => x.check).length;
    this.isDataListChecked = Boolean(this.numDataListChecked)
  }
  searchChange() {
    this.currentPage = 1
    this.page = Array.from({ length: Math.ceil(this.evaluation_cycleListFilter().length / 10) }, (_, i) => i + 1);
    this.dataListCheck()
  }
  evaluation_cycleListFilter() {
    return this.evaluation_cyclelist.filter(x => {
      const data = x.data
      const match = data.evaluationRoundId.toLowerCase().includes(this.search.toLowerCase()) || data.tdesc.toLowerCase().includes(this.search.toLowerCase()) || data.edesc.toLowerCase().includes(this.search.toLowerCase());
      return match;
    });
  }
  setData(data?: DataModel) {
    this.dataSelect = JSON.parse(JSON.stringify(data || {
      evaluationRoundId: "",
      tdesc: "",
      edesc: "",
      apsyear: "",
      apsPeriodStart: "",
      apsPeriodEnd: "",
      statusCode: new MyStatusCodeModel({})
    }));
  }
  addevaluation_cycle() {
    const body = new MyEvaluationCycleModel({ evaluationRoundId: this.dataSelect.evaluationRoundId, tdesc: this.dataSelect.tdesc, edesc: this.dataSelect.edesc, apsyear: this.dataSelect.apsyear, apsPeriodStart: this.dataSelect.apsPeriodStart, apsPeriodEnd: this.dataSelect.apsPeriodEnd, statusCode: this.dataSelect.statusCode })
    this.dataLoading = true
    this.evaluationCycleService.post(body).subscribe({
      next: response => {
        if (response.success) {
          this.showAlert(response.message, 'success')
          this.getEvaluationCycleList()
        } else {
          this.dataLoading = false
          this.showAlert(response.message, 'error')
          this.cdr.detectChanges()
        }
      }, error: error => {
        this.showAlert(error.message, 'error')
        this.dataLoading = false
        this.cdr.detectChanges()
      }
    })
  }

  deleteevaluation_cycle() {
    let body: EvaluationCycleModel | EvaluationCycleModel[] = []
    if (this.dataSelect.evaluationRoundId) {
      body = new MyEvaluationCycleModel({ evaluationRoundId: this.dataSelect.evaluationRoundId, tdesc: this.dataSelect.tdesc, edesc: this.dataSelect.edesc, apsyear: this.dataSelect.apsyear, apsPeriodStart: this.dataSelect.apsPeriodStart, apsPeriodEnd: this.dataSelect.apsPeriodEnd, statusCode: this.dataSelect.statusCode })
    } else {
      body = this.evaluation_cyclelist.filter(x => x.check).map(x => new MyEvaluationCycleModel({ evaluationRoundId: x.data.evaluationRoundId, tdesc: x.data.tdesc, edesc: x.data.edesc, apsyear: x.data.apsyear, apsPeriodStart: x.data.apsPeriodStart, apsPeriodEnd: x.data.apsPeriodEnd, statusCode: x.data.statusCode }))
    }
    this.dataLoading = true
    this.evaluationCycleService.delete(body).subscribe({
      next: response => {
        if (response.success) {
          this.showAlert(response.message, 'success')
          this.getEvaluationCycleList()
        } else {
          this.dataLoading = false
          this.showAlert(response.message, 'error')
          this.dataLoading = false
          this.cdr.detectChanges()
        }
      }, error: error => {
        this.showAlert(error.message, 'error')
        this.dataLoading = false
        this.cdr.detectChanges()
      }
    })
  }

  showAlert(text: string, type: 'success' | 'error') {
    this.toastr[type](text, 'แจ้งเตือน', {
      timeOut: 3000,
      positionClass: 'toast-top-right',
    });
  }

  clear() {
    if (this.modalStatus == 'add') {
      this.setData()
    } else if (this.modalStatus == 'edit') {
      this.setData(new MyEvaluationCycleModel({ evaluationRoundId: this.dataSelect.evaluationRoundId }))
    }
  }

  openModal(id: string,evaluationRoundId:string) {
    this.evaluationRoundId = ''
    if(id == 'evaluation-cycle-person-modal'){
      this.evaluationRoundId = evaluationRoundId
      this.evaluationRoundIdChange.emit(evaluationRoundId)
      setTimeout(() => {
        document.getElementById(id)?.classList.add('open');
      document.getElementById(id)?.classList.remove('hidden');
        document.getElementById(id)?.setAttribute('aria-overlay', 'false');
      }, 10);
    }
  }

  

}


